//=============================================================================
// (c) Copyright 2011 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// Module Name: TxTest.c - Test SJA1000 CAN Controller in Transmit Mode
//
// Environment: User mode
//    
// Depedency :	DDK Library installed.
//				
//				
// Description: This sample code demonstrates the usage of  the JanusMM CAN driver
//				on Windows XP OS. This code provides the usage of the driver in
//				receive mode to read data coming from the bus.
//				The sample code demonstrates how to Open the device and set the
//				parameters like Baud rate,base address and write to the hardware.
//=============================================================================

#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

#pragma warning(disable:4201)  // disable nameless struct/union warnings
#include <winioctl.h>
#include <setupapi.h>
#pragma warning(default:4201)

#include "public.h"

#define BUFSIZE 20//(1024 * 11)

static
ULONG   bufsize = BUFSIZE;

#if defined(BUILT_IN_DDK)

#define scanf_s scanf
#define sscanf_s sscanf

#endif

HANDLE
OpenDevice(
    IN CONST GUID * InterfaceGuid,
    IN ULONG        FileFlagOptions
    )
;


void print_usage();

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
int
__cdecl
main(
    __in int argc,
    __in_ecount(argc) char* argv[]
    )
{
    HANDLE      hISAdevice;
    HANDLE      hPCIdevice;
    PDWORD      inbuf;
    ULONG       count;
    DWORD       numread;
    OVERLAPPED  ol = {0};
    ULONG       offset;
    ULONG       length;
    DWORD       numwritten,dwError=0,Mem_Addr = 0;
    BOOL        okay;
    int       i,k;
    BOOL        hasErrors = FALSE;
    PVERSION_INFO inbuf1 = NULL;

	UCHAR		FrameFormat, Identifier, DLC;//,DLC1,x;
	UCHAR		Data[20],cTx = 1 , loop = 1 ;
	UCHAR		tcTx = 1,increment=0;

	PUCHAR		outbuf1;
	UCHAR		rxCount,x,fileXfr = 0;

	ULONG iCount = 0, j = 0;

    FILE *fd = NULL, *fp = NULL;
    DWORD err;
    int c,ret;

	int loopcount = 0, index=0, lCount = 0;
	int s, opt, dlc = 0, rtr = 0, extended = 0;
	int verbose = 0,pattern = 0, infinite = 0;
	char *data[8];
	char *inputFile = "input.txt";
	char *can_name;
	long bit_rate, identifier;
	//long address = 0xD7200;
	long address = 0xD7200;

	int dataSize = 0;
	char * pEnd;


	if(argc < 2)
	{
		print_usage();
		return 0;

	}

	can_name = argv[1];

	/// parsing the options part
	for( i = 2; i < argc; i++ )
	{
		if( argv[i][0] == '-' )
		{
			switch( argv[i][1] )
			{
				case 'a':		//	bit rate
				{
					address = strtol (argv[++i],&pEnd,16);
					break;
				}
				case 'b':		//	bit rate
				{
					bit_rate = atoi(argv[++i]);
					break;
				}
				case 'l':		//	Loop Count
				{
					loopcount=atoi(argv[++i]);
					break;
				}
				case 'i':		// Identifier
				{
					identifier=atoi(argv[++i]);
					break;
				}
				case 'e' :		// Extended frame
				{
					extended = 1;
					break;
				}
				case 'r' :		// Remote
				{
					rtr = 1;
					break;
				}
				case 'p' :		// Pattern
				{
					pattern = 1;
					break;
				}
				case 'o' :		// Input File name
				{
					inputFile = argv[++i];
					fileXfr = 1;

					break;
				}
				case 'h':
				{
					print_usage();
				}
				default:
				{
					printf("Default is activated ...\r\n");
				}
			}
		}
		else
		{
			data[dataSize] = argv[i];
			dataSize++;
		}
	}

	if( ( ret = strcmp(can_name,"canA") ) == 0) {

		hISAdevice = OpenDevice(&GUID_DEVINTERFACE_CAN,
								FILE_FLAG_OVERLAPPED );

		if (hISAdevice == INVALID_HANDLE_VALUE) {
			puts("(ISA) Can't open S5933DK1 device");
			exit(1);
		}
	}
	else {
		hISAdevice = OpenDevice(&GUID_DEVINTERFACE_SJA1000,
								FILE_FLAG_OVERLAPPED );

		if (hISAdevice == INVALID_HANDLE_VALUE) {
			puts("(ISA) Can't open S5933DK1 device");
			exit(1);
		}
	}

    inbuf = (PDWORD) malloc(bufsize);
    if (inbuf == NULL) {
        puts("memory allocation for input buffer failed\n");
        exit(1);
    }

    okay = DeviceIoControl( hISAdevice,
							IOCTL_SET_BAUD_RATE,
							&bit_rate,
							sizeof(bit_rate),
							inbuf,
							bufsize,
							&numread,
							NULL );

	if(!okay)
		printf("CAN Controller baud rate Failed ...\r\n");

    okay = DeviceIoControl( hISAdevice,
							IOCTL_SET_MEM,
							&address,
							sizeof(address),
							inbuf,
							bufsize,
							&numread,
							NULL );

	if(!okay)
		printf("CAN Controller Initialization Failed ...\r\n");


	inbuf1 = malloc(sizeof(VERSION_INFO));

	if(inbuf1 == NULL)
		printf("Out Buffer is NULL ....\r\n");

	outbuf1 = (PCHAR) malloc(12);
	if (outbuf1 == NULL) {
		puts("memory allocation for output buffer failed\n");
		exit(1);
	}

	if(loopcount == 0) 
		infinite = 1;

	if(fileXfr == 1)
	{
		while( infinite || loopcount-- )
		{

			fp = fopen(inputFile/*"input.txt"*/,"r");

			if( fp == NULL )
			{
				printf("ERROR :: Opening File gives Error \r\n");
			}
			if(rtr == 1)
				outbuf1[0] = (UCHAR)( extended | 0x2 );
			else
				outbuf1[0] = (UCHAR)( extended );

			outbuf1[1] = (UCHAR)identifier;

			do 
			{

				c = fgetc(fp);

				outbuf1[iCount+3] = (unsigned char)c;

				iCount++;

				if((iCount == 8) | (c == EOF))
				{
					outbuf1[2] = (UCHAR)iCount;

					memset(inbuf, 0, bufsize);

					okay = DeviceIoControl( hISAdevice,
										IOCTL_WRITE,
										outbuf1,
										iCount+3,
										inbuf,
										bufsize,
										&numread,
										NULL );

					if(!okay)
					{
						printf("DeviceIoControl Failed ... \r\n");
					}

					Sleep(100); // milli-sec

					iCount = 0;

				} // if


			} while (c != EOF);

			fclose(fp);

		}
	}
	else	// Raw-Data transfer
	{

		if(pattern == 1)
			for(i = 0; i < dataSize; i++)
				outbuf1[i+3] = (unsigned char)i;


		while( infinite || loopcount-- )
		{

			if(rtr == 1)
				outbuf1[0] = (UCHAR)( extended | 0x2 );
			else
				outbuf1[0] = (UCHAR)( extended );

			outbuf1[1] = (UCHAR)identifier;

		  	for(i = 0; i < dataSize; i++)
		  	{
				if(pattern == 0)
					outbuf1[i+3] = (unsigned char)atoi(data[i]);
				else
					outbuf1[i+3] = outbuf1[i+3] + 1;
			}

			outbuf1[2] = (UCHAR)dataSize;

			memset(inbuf, 0, bufsize);

			okay = DeviceIoControl( hISAdevice,
										IOCTL_WRITE,
										outbuf1,
										dataSize+3,
										inbuf,
										bufsize,
										&numread,
										NULL );

			if(!okay)
			{
				printf("DeviceIoControl Failed ... \r\n");
			}

			if(outbuf1[0] == 1)
				printf("[E]   ");
			else
				printf("[S]   ");

			printf("[ID %d]  ",outbuf1[1]);


			for (j=3; j < (unsigned char)(dataSize+3); j++) 
			{
				printf("%3d ",outbuf1[j]);
			}

			printf("\r\n");

			Sleep(2); // milli-sec

		} 

	}

    CloseHandle(ol.hEvent);
    free(inbuf);
    free(outbuf1);
    CloseHandle(hISAdevice);

    exit((hasErrors)?1:0);
}


HANDLE
OpenDevice(
    IN CONST GUID * InterfaceGuid,
    IN ULONG        FileFlagOptions
    )
{
    SP_DEVICE_INTERFACE_DATA DeviceInterfaceData;
    SP_DEVINFO_DATA DeviceInfoData;
    HDEVINFO hDevInfo;
    PSP_DEVICE_INTERFACE_DETAIL_DATA pDeviceInterfaceDetail = NULL;
    HANDLE hDevice;

    ULONG size;
    int count, i, index;
    BOOL status = TRUE;
    TCHAR *DeviceName = NULL;
    TCHAR *DeviceLocation = NULL;


    //
    //  Retreive the device information for all PLX devices.
    //
    hDevInfo =     SetupDiGetClassDevs(InterfaceGuid,
                                                NULL,
                                                NULL,
                                                DIGCF_DEVICEINTERFACE |
                                                DIGCF_PRESENT );
    if(INVALID_HANDLE_VALUE == hDevInfo) {
        printf("No AMCC devices are present and enabled in the system\n");
        return INVALID_HANDLE_VALUE;
    }
    //
    //  Initialize the SP_DEVICE_INTERFACE_DATA Structure.
    //
    DeviceInterfaceData.cbSize  = sizeof(SP_DEVICE_INTERFACE_DATA);

    //
    //  Determine how many devices are present.
    //
    count = 0;
    while(SetupDiEnumDeviceInterfaces(hDevInfo,
                                    NULL,
                                    InterfaceGuid,
                                    count++,  //Cycle through the available devices.
                                    &DeviceInterfaceData));

    //
    // Since the last call fails when all devices have been enumerated,
    // decrement the count to get the true device count.
    //
    count--;

    //
    //  If the count is zero then there are no devices present.
    //
    if(count == 0)
    {
        printf("No AMCC devices are present and enabled in the system.\n");
        goto End;
    }


    //
    //  Initialize the appropriate data structures in preparation for
    //  the SetupDi calls.
    //
    DeviceInterfaceData.cbSize = sizeof(SP_DEVICE_INTERFACE_DATA);
    DeviceInfoData.cbSize = sizeof(SP_DEVINFO_DATA);


    //
    //  Loop through the device list to allow user to choose
    //  a device.  If there is only one device, select it
    //  by default.
    //
    i = 0;
    while(SetupDiEnumDeviceInterfaces(hDevInfo,
                                NULL,
                                (LPGUID)InterfaceGuid,
                                i,
                                &DeviceInterfaceData))
    {

        //
        // Determine the size required for the DeviceInterfaceData
        //
        SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                        &DeviceInterfaceData,
                                        NULL,
                                        0,
                                        &size,
                                        NULL);

        if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
        {
            printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
            goto End;
        }
        pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
        if(!pDeviceInterfaceDetail)
        {
            printf("Insufficient memory.\n");
            goto End;
        }

        //
        // Initialize structure and retrieve data.
        //
        pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
        status = SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                    &DeviceInterfaceData,
                                    pDeviceInterfaceDetail,
                                    size,
                                    NULL,
                                    &DeviceInfoData);

        free(pDeviceInterfaceDetail);
        pDeviceInterfaceDetail = NULL;

        if(!status)
        {
            printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
            goto End;
        }

        //
        //  Get the Device Name
        //  Calls to SetupDiGetDeviceRegistryProperty require two consecutive
        //  calls, first to get required buffer size and second to get
        //  the data.
        //
        SetupDiGetDeviceRegistryProperty(hDevInfo,
                                        &DeviceInfoData,
                                        SPDRP_DEVICEDESC,
                                        NULL,
                                        (PBYTE)DeviceName,
                                        0,
                                        &size);

        if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
        {
            printf("SetupDiGetDeviceRegistryProperty failed, Error: %d", GetLastError());
            goto End;
        }

        DeviceName = (TCHAR*) malloc(size);
        if(!DeviceName)
        {
            printf("Insufficient memory.\n");
            goto End;
        }

        status = SetupDiGetDeviceRegistryProperty(hDevInfo,
                                        &DeviceInfoData,
                                        SPDRP_DEVICEDESC,
                                        NULL,
                                        (PBYTE)DeviceName,
                                        size,
                                        NULL);
        if(!status)
        {
            printf("SetupDiGetDeviceRegistryProperty failed, Error: %d", GetLastError());
            free(DeviceName);
            goto End;
        }

        //
        //  Now retrieve the Device Location.
        //
        SetupDiGetDeviceRegistryProperty(hDevInfo,
                                        &DeviceInfoData,
                                        SPDRP_LOCATION_INFORMATION,
                                        NULL,
                                        (PBYTE)DeviceLocation,
                                        0,
                                        &size);

        if(GetLastError() == ERROR_INSUFFICIENT_BUFFER)
        {
            DeviceLocation = (TCHAR*) malloc(size);

            if (DeviceLocation != NULL)
            {
                status = SetupDiGetDeviceRegistryProperty(hDevInfo,
                                                          &DeviceInfoData,
                                                          SPDRP_LOCATION_INFORMATION,
                                                          NULL,
                                                          (PBYTE)DeviceLocation,
                                                          size,
                                                          NULL);
                if(!status)
                {
                    free(DeviceLocation);
                    DeviceLocation = NULL;
                }
            }

        } else {
            DeviceLocation = NULL;
        }

        //
        // If there is more than one device print description.
        //
        if(count > 1)
        {
            printf("%d- ", i);
        }

        printf("%s\n", DeviceName);

        if(DeviceLocation)
        {
            printf("        %s\n", DeviceLocation);
        }

        free(DeviceName);
        free(DeviceLocation);

        i++; // Cycle through the available devices.
    }

    //
    //  Select device.
    //
    index = 0;
    if(count > 1)
    {
        printf("\nSelect Device: ");
        (void)scanf_s("%d", &index);
    }

    //
    //  Get information for specific device.
    //

    status = SetupDiEnumDeviceInterfaces(hDevInfo,
                                    NULL,
                                    (LPGUID)InterfaceGuid,
                                    index,
                                    &DeviceInterfaceData);

    if(!status)
    {
        printf("SetupDiEnumDeviceInterfaces failed, Error: %d", GetLastError());
        goto End;
    }
    //
    // Determine the size required for the DeviceInterfaceData
    //
    SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                    &DeviceInterfaceData,
                                    NULL,
                                    0,
                                    &size,
                                    NULL);

    if(GetLastError() != ERROR_INSUFFICIENT_BUFFER)
    {
        printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
        goto End;
    }
    pDeviceInterfaceDetail = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(size);
    if(!pDeviceInterfaceDetail)
    {
        printf("Insufficient memory.\n");
        goto End;
    }

    //
    // Initialize structure and retrieve data.
    //
    pDeviceInterfaceDetail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);
    status = SetupDiGetDeviceInterfaceDetail(hDevInfo,
                                &DeviceInterfaceData,
                                pDeviceInterfaceDetail,
                                size,
                                NULL,
                                &DeviceInfoData);
    if(!status)
    {
        printf("SetupDiGetDeviceInterfaceDetail failed, Error: %d", GetLastError());
        goto End;
    }

    //
    //  Get handle to device.
    //
    hDevice = CreateFile(pDeviceInterfaceDetail->DevicePath,
                            GENERIC_READ|GENERIC_WRITE,
                            FILE_SHARE_READ | FILE_SHARE_WRITE,
                            NULL,
                            OPEN_EXISTING,
                            FileFlagOptions,
                            NULL
                            );

    if(hDevice == INVALID_HANDLE_VALUE)
    {
        printf("CreateFile failed.  Error:%d", GetLastError());
    }

    free(pDeviceInterfaceDetail);
    SetupDiDestroyDeviceInfoList(hDevInfo);

    return hDevice;

End:
    if(pDeviceInterfaceDetail) {
        free(pDeviceInterfaceDetail);
    }
    SetupDiDestroyDeviceInfoList(hDevInfo);
    return INVALID_HANDLE_VALUE;
}

void print_usage()
{

	printf(
		"Usage: canSend.exe [<can-interface>] [Options] <can-msg>\n"
		"<can-msg> can consist of up to 8 bytes given as a space separated list\n"
		"Options:\n"
		" -i    [ ex: -i ID ] CAN Identifier (default = 1)\n"
		" -r    send remote request\n"
		" -e    send extended frame\n"
		" -l    [ ex : -l COUNT ]	send message COUNT times\n"
		" -p    send message data in incremental pattern\n"
		" -b    [ ex : -b br_value ] baud-rate (default = 1000000) \n"
		" -a    [ ex : -a address  ] address (default = 0xD7000) \n"
		" -o    <filename>\t\t"    "Input filename\n"
		" -h,   --help		this help\n"
		);

}



